package org.freecompany.imp;

import org.freecompany.xmlwriter.XmlWriter;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.regex.Pattern;

import static java.util.regex.Pattern.compile;
import static java.util.Arrays.asList;
import static org.apache.tools.ant.Project.MSG_DEBUG;
import static org.freecompany.brimstone.module.Module.alias;
import static org.freecompany.imp.Constants.PROPERTY_GROUP;
import static org.freecompany.imp.Constants.PROPERTY_JAVADOC;

public class ModuleTask extends Task {

	public static final String NAMESPACE = "http://www.freecompany.org/namespace/brimstone/module";
	
	protected String name;
	protected String version;
	protected String group;
	protected String run;
	protected String test;
	protected File destination;

	public void execute() throws BuildException {
		if ( name == null) throw new BuildException( "Attribute 'name' is required.");
		if ( group == null) throw new BuildException( "Attribute 'group' is required.");
		if ( version == null) throw new BuildException( "Attribute 'version' is required.");
		if ( run == null) throw new BuildException( "Attribute 'run' is required.");
		if ( test == null) throw new BuildException( "Attribute 'test' is required.");

		final Collection< String> components = new ArrayList< String>();
		components.addAll( Util.getComponents( getProject()));
		components.addAll( list( run));

		final Collection< String> testComponents = new ArrayList< String>();
		testComponents.addAll( Util.getTestComponents( getProject()));
		testComponents.addAll( list( test));
		
		String module = name + "-" + version + ".module";
		log( "Creating module: " + module);
		try {
			File file = new File( destination, module);
			file.createNewFile();
			XmlWriter writer = new XmlWriter( new OutputStreamWriter( new FileOutputStream( file)));
			writer.setIndented( true);
			writer.startPrefixMapping( "", NAMESPACE);
			writer.startElement( NAMESPACE, "module");
			writer.createElement( NAMESPACE, "name", name);
			writer.createElement( NAMESPACE, "version", version);

			final String javadoc = getProject().getProperty( PROPERTY_JAVADOC);
			if ( javadoc != null) writer.createElement( NAMESPACE, "javadoc", javadoc);

			for ( String component : components) {
				log( "Adding component: " + component, MSG_DEBUG);
				writer.startElement( NAMESPACE, "component");
				writer.createElement( NAMESPACE, "uri", component);
				writer.createElement( NAMESPACE, "alias", alias( component));
				writer.endElement( NAMESPACE, "component");
			}
			for ( Dependency dependency : Util.getPackage( getProject())) writer.createElement( NAMESPACE, "dependency", dependency.getUri());

			if ( !testComponents.isEmpty()) {
				writer.startElement( NAMESPACE, "test");
				for ( String testComponent : testComponents) {
					log( "Adding test component: " + testComponent, MSG_DEBUG);
					writer.startElement( NAMESPACE, "component");
					writer.createElement( NAMESPACE, "uri", testComponent);
					writer.createElement( NAMESPACE, "alias", alias( testComponent));
					writer.endElement( NAMESPACE, "component");
					for ( Dependency dependency : Util.getTest( getProject())) writer.createElement( NAMESPACE, "dependency", dependency.getUri());
				}
				writer.endElement( NAMESPACE, "test");
			}

			writer.endElement( NAMESPACE, "module");
			writer.close();
		} catch ( Exception e) {
			throw new BuildException( "Error creating module.", e);
		}
	}

	protected List< String> list( final String reference) {
		final Pattern pattern = compile( ".*/(.*)");
		final String prefix = "urn:brimstone:" + getProject().getProperty( PROPERTY_GROUP) + "/";

		final FileSet set = ( FileSet) getProject().getReference( reference);
		if ( set == null) return Collections.< String>emptyList();

		final List< String> list = new ArrayList< String>();
		for ( String path : set.getDirectoryScanner( getProject()).getIncludedFiles()) {
			list.add( prefix + pattern.matcher( path).replaceFirst( "$1"));
		}
		return list;
	}

	public void setName( String name) { this.name = name; }
	public void setVersion( String version) { this.version = version; }
	public void setGroup( String group) { this.group = group; }
	public void setRun( String run) { this.run = run; }
	public void setTest( String test) { this.test = test; }
	public void setDestination( File destination) { this.destination = destination; }
}
